/*
 * JLab
 * Copyright 2007/2008, Volker Raum, Erlangen, Germany
 *                2008, Marco Kubon, Bruchsal, Germany
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */package de.jlab.external.measurement.devices.voltcraftvc98;

import java.util.ArrayList;
import java.util.List;

import de.jlab.external.measurement.converters.MeasurementConverter;
import de.jlab.external.measurement.model.data.CombiMeasurement;
import de.jlab.external.measurement.model.data.Measurement;
import de.jlab.external.measurement.model.data.Measurement.ACDC;
import de.jlab.external.measurement.model.data.Measurement.MEASUREMENT;
import de.jlab.external.measurement.model.data.Measurement.UNIT;

public class ConverterVC98 implements MeasurementConverter {
	final boolean rlDEBUG = true;
	final byte typeAndUnitStart = 9;
	final byte typeAndUnitEnd = 10;
	final byte valueStart = 3;
	final byte valueEnd = 8;

	StringBuffer buffer = new StringBuffer();

	/* Beispiel Voltcraft VC98:
	 * 
	 * im Uhrzeigersinn beginnend bei DCV 400mV
	 * 
	 * DCV- 00.1mV
	 * DCV  .001 V
	 * DCV  0.01 V
	 * DCV  00.1 V
	 * DCV  001  V
	 * 
	 * ACV  230  V
	 * ACV  00.1 V
	 * ACV  0.01 V
	 * ACV  .006 V
	 * 
	 * DCA  00.1uA
	 * DCA  .001mA
	 * DCA  0.01mA
	 * DCA  00.1mA
	 * DCA  .001 A
	 * DCA  0.01 A
	 * 
	 * ACA  0.01 A
	 * ACA  .001 A
	 * ACA  00.1mA
	 * ACA  0.01mA
	 * ACA  .001mA
	 * 
	 * hfe  153
	 * 
	 * FRQ 1.250kz
	 * 
	 * CAP 1911 nF
	 * 
	 * DIO  171 mV
	 * 
	 * OHM  00.3 
	 * OHM  .001k
	 * OHM  0.01k
	 * OHM  00.0k
	 * OHM  .000M
	 * OHM  .000M
	 * 
	 */
	@Override
	public List<Measurement> convertData(byte[] dataFromDevice, int length) {
		String stringData = new String(dataFromDevice, 0, length);
		List<Measurement> newList = new ArrayList<Measurement>();
		analyseBuffer(stringData, newList);
		return newList;
	}

	public void analyseBuffer(String buffer, List<Measurement> measurements) {
		CombiMeasurement created = new CombiMeasurement();

		// Unit(k, m, u, M etc.) and Measurement (V, A, Hz)

		String typeAndUnit = buffer.substring(typeAndUnitStart, typeAndUnitEnd)
				.trim();

		if (rlDEBUG) {
			System.out.println("B : " + buffer);
		}
		if (rlDEBUG) {
			System.out.println("TU: " + typeAndUnit);
		}

		if (typeAndUnit.contains("V"))
			created.setMeasurement(MEASUREMENT.VOLTAGE);
		else if (typeAndUnit.contains("A"))
			created.setMeasurement(MEASUREMENT.CURRENT);
		else if (buffer.startsWith("OHM"))
			created.setMeasurement(MEASUREMENT.RESISTANCE);
		else if (buffer.startsWith("FRQ"))
			created.setMeasurement(MEASUREMENT.FREQUENCY);
		else if (buffer.startsWith("CAP"))
			created.setMeasurement(MEASUREMENT.CAPACITY);
		else if (buffer.startsWith("hfe"))
			created.setMeasurement(MEASUREMENT.HFE);
		else if (buffer.startsWith("DIO"))
			created.setMeasurement(MEASUREMENT.DIODE);
		else if (typeAndUnit.isEmpty())
			created.setMeasurement(MEASUREMENT.UNKNOWN);
		else
			return;

		if (typeAndUnit.startsWith("m"))
			created.setUnit(UNIT.MILLI);
		if (typeAndUnit.startsWith("M"))
			created.setUnit(UNIT.MEGA);
		if (typeAndUnit.startsWith("k"))
			created.setUnit(UNIT.KILO);
		if (typeAndUnit.startsWith("n")) // nF
			created.setUnit(UNIT.NANO);
		if (typeAndUnit.startsWith("u")) // uF
			created.setUnit(UNIT.MICRO);

		if (buffer.startsWith("AC"))
			created.setAcdc(ACDC.AC);
		else if (buffer.startsWith("DC"))
			created.setAcdc(ACDC.DC);

		// Measurement Value
		String valueString = buffer.substring(valueStart, valueEnd).trim();

		if (rlDEBUG) {
			System.out.println("V : " + valueString);
		}

		// Out Of Range ?
		if ((valueString.contains("OL")) || (valueString.contains("O.L"))) // O.L
																			// ->
																			// 200
																			// OHM
																			// Range
			created.setValue(99999);
		else {
			try {
				created.setValue(Double.parseDouble(valueString));
			} catch (NumberFormatException e) {
				// Error in conversion => deliver NO measurement
				return;
			}
		}

		// Add one measurement
		measurements.add(created);

	}

}
